"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeConsumerPact = void 0;
var ffi_1 = require("../ffi");
var types_1 = require("../ffi/types");
var logger_1 = require("../logger");
var checkErrors_1 = require("./checkErrors");
var handleStringResult = function (res) {
    if (res.tag !== 0) {
        logger_1.logErrorAndThrow("invalid multipart body specified: " + res.ok);
    }
    return true;
};
var makeConsumerPact = function (consumer, provider, version, logLevel) {
    if (version === void 0) { version = 3; }
    if (logLevel === void 0) { logLevel = logger_1.getLogLevel(); }
    var lib = ffi_1.getFfiLib(logLevel);
    var pactPtr = lib.pactffi_new_pact(consumer, provider);
    if (!lib.pactffi_with_specification(pactPtr, version)) {
        throw new Error("Unable to set core spec version. The pact FfiSpecificationVersion '" + version + "' may be invalid (note this is not the same as the pact spec version)");
    }
    return {
        createMockServer: function (address, requestedPort, tls) {
            if (tls === void 0) { tls = false; }
            var port = lib.pactffi_create_mock_server_for_pact(pactPtr, address + ":" + (requestedPort ? requestedPort : 0), tls);
            var error = Object.keys(types_1.CREATE_MOCK_SERVER_ERRORS).find(function (key) { return types_1.CREATE_MOCK_SERVER_ERRORS[key] === port; });
            if (error) {
                if (error === 'ADDRESS_NOT_VALID') {
                    logger_1.logErrorAndThrow("Unable to start mock server at '" + address + "'. Is the address and port valid?");
                }
                if (error === 'TLS_CONFIG') {
                    logger_1.logErrorAndThrow("Unable to create TLS configuration with self-signed certificate");
                }
                logger_1.logCrashAndThrow("The pact core couldn't create the mock server because of an error described by '" + error + "'");
            }
            if (port <= 0) {
                logger_1.logCrashAndThrow("The pact core returned an unhandled error code '" + port + "'");
            }
            return port;
        },
        mockServerMatchedSuccessfully: function (port) {
            return lib.pactffi_mock_server_matched(port);
        },
        mockServerMismatches: function (port) {
            var results = JSON.parse(lib.pactffi_mock_server_mismatches(port));
            return results.map(function (result) { return (__assign(__assign({}, result), ('mismatches' in result
                ? {
                    mismatches: result.mismatches.map(function (m) {
                        return typeof m === 'string' ? JSON.parse(m) : m;
                    }),
                }
                : {}))); });
        },
        cleanupMockServer: function (port) {
            return checkErrors_1.wrapWithCheck(function (port) { return lib.pactffi_cleanup_mock_server(port); }, 'cleanupMockServer')(port);
        },
        writePactFile: function (port, dir, merge) {
            if (merge === void 0) { merge = true; }
            var result = lib.pactffi_write_pact_file(port, dir, !merge);
            switch (result) {
                case types_1.FfiWritePactResponse.SUCCESS:
                    return;
                case types_1.FfiWritePactResponse.UNABLE_TO_WRITE_PACT_FILE:
                    logger_1.logErrorAndThrow('The pact core was unable to write the pact file');
                case types_1.FfiWritePactResponse.GENERAL_PANIC:
                    logger_1.logCrashAndThrow('The pact core panicked while writing the pact file');
                case types_1.FfiWritePactResponse.MOCK_SERVER_NOT_FOUND:
                    logger_1.logCrashAndThrow('The pact core was asked to write a pact file from a mock server that appears not to exist');
                default:
                    logger_1.logCrashAndThrow("The pact core returned an unknown error code (" + result + ") instead of writing the pact");
            }
        },
        addMetadata: function (namespace, name, value) {
            return lib.pactffi_with_pact_metadata(pactPtr, namespace, name, value);
        },
        newInteraction: function (description) {
            var interactionPtr = lib.pactffi_new_interaction(pactPtr, description);
            return checkErrors_1.wrapAllWithCheck({
                uponReceiving: function (description) {
                    return lib.pactffi_upon_receiving(interactionPtr, description);
                },
                given: function (state) {
                    return lib.pactffi_given(interactionPtr, state);
                },
                givenWithParam: function (state, name, value) {
                    return lib.pactffi_given_with_param(interactionPtr, state, name, value);
                },
                withRequest: function (method, path) {
                    return lib.pactffi_with_request(interactionPtr, method, path);
                },
                withQuery: function (name, index, value) {
                    return lib.pactffi_with_query_parameter(interactionPtr, name, index, value);
                },
                withRequestHeader: function (name, index, value) {
                    return lib.pactffi_with_header(interactionPtr, types_1.INTERACTION_PART_REQUEST, name, index, value);
                },
                withRequestBody: function (body, contentType) {
                    return lib.pactffi_with_body(interactionPtr, types_1.INTERACTION_PART_REQUEST, contentType, body);
                },
                withRequestMultipartBody: function (contentType, filename, mimePartName) {
                    var res = lib.pactffi_with_multipart_file(interactionPtr, types_1.INTERACTION_PART_REQUEST, contentType, filename, mimePartName);
                    return handleStringResult(res);
                },
                withResponseHeader: function (name, index, value) {
                    return lib.pactffi_with_header(interactionPtr, types_1.INTERACTION_PART_RESPONSE, name, index, value);
                },
                withResponseBody: function (body, contentType) {
                    return lib.pactffi_with_body(interactionPtr, types_1.INTERACTION_PART_RESPONSE, contentType, body);
                },
                withResponseMultipartBody: function (contentType, filename, mimePartName) {
                    var res = lib.pactffi_with_multipart_file(interactionPtr, types_1.INTERACTION_PART_RESPONSE, contentType, filename, mimePartName);
                    return handleStringResult(res);
                },
                withStatus: function (status) {
                    return lib.pactffi_response_status(interactionPtr, status);
                },
            });
        },
    };
};
exports.makeConsumerPact = makeConsumerPact;
//# sourceMappingURL=index.js.map